@@ -82,58 +82,18 @@
 
 use bytes::{BufMut, Bytes, BytesMut};
 use env_logger::filter::{Builder as FilterBuilder, Filter};
-#[cfg(all(not(feature = "tls"), target_os = "android"))]
-use lazy_static::lazy_static;
 use log::{LevelFilter, Log, Metadata, SetLoggerError};
-#[cfg(target_os = "android")]
-use std::os::unix::net::UnixDatagram;
 use std::{fmt, io, iter::FromIterator, time::SystemTime};
 use thiserror::Error;
 
-#[cfg(target_os = "android")]
-const LOGDW: &str = "/dev/socket/logdw";
+#[allow(dead_code)]
+#[cfg(not(target_os = "windows"))]
+mod logd;
+mod thread;
 
 /// Max log entry len
 const LOGGER_ENTRY_MAX_LEN: usize = 5 * 1024;
 
-#[cfg(all(feature = "tls", target_os = "android"))]
-thread_local! {
-     static SOCKET: UnixDatagram = {
-        let socket = UnixDatagram::unbound().expect("Failed to create socket");
-        socket.connect(LOGDW).expect("Failed to connect to /dev/socket/logdw");
-        socket
-    };
-}
-#[cfg(all(not(feature = "tls"), target_os = "android"))]
-lazy_static! {
-    static ref SOCKET: UnixDatagram = {
-        let socket = UnixDatagram::unbound().expect("Failed to create socket");
-        socket.connect(LOGDW).expect("Failed to connect to /dev/socket/logdw");
-        socket
-    };
-}
-
-mod thread {
-    #[cfg(unix)]
-    #[inline]
-    pub fn id() -> usize {
-        unsafe { libc::pthread_self() as usize }
-    }
-
-    #[cfg(windows)]
-    #[inline]
-    pub fn id() -> usize {
-        unsafe { winapi::um::processthreadsapi::GetCurrentThreadId() as usize }
-    }
-
-    #[cfg(target_os = "redox")]
-    #[inline]
-    pub fn id() -> usize {
-        // Each thread has a separate pid on Redox.
-        syscall::getpid().unwrap()
-    }
-}
-
 /// Error
 #[derive(Error, Debug)]
 pub enum Error {
@@ -226,19 +186,38 @@ impl From<Buffer> for u8 {
     }
 }
 
+/// Tag mode
+#[derive(Debug, Clone)]
+enum TagMode {
+    /// Use the records target metadata as tag
+    Target,
+    /// Use root module as tag. The target field contains the module path
+    /// if not overwritten. Use the root module as tag. e.g a target of
+    /// `crate::module::submodule` will be `crate`.
+    TargetStrip,
+    /// Custom fixed tag string
+    Custom(String),
+}
+
+impl Default for TagMode {
+    fn default() -> Self {
+        TagMode::TargetStrip
+    }
+}
+
 /// Builder for initializing logger
 ///
 /// The builder is used to initialize the logging framework for later use.
 /// It provides
 #[derive(Default)]
-pub struct Builder<'a> {
+pub struct Builder {
     filter: FilterBuilder,
-    tag: Option<&'a str>,
-    prepend_module: Option<bool>,
+    tag: TagMode,
+    prepend_module: bool,
     buffer: Option<Buffer>,
 }
 
-impl<'a> Builder<'a> {
+impl<'a> Builder {
     /// Initializes the log builder with defaults.
     ///
     /// # Examples
@@ -250,18 +229,16 @@ impl<'a> Builder<'a> {
     /// # use android_logd_logger::Builder;
     ///
     /// let mut builder = Builder::new();
-    ///
-    /// builder.filter(None, LevelFilter::Info)
-    ///        .init();
+    /// builder.filter(None, LevelFilter::Info).init();
     /// ```
     ///
     /// [`filter`]: #method.filter
-    pub fn new() -> Builder<'a> {
+    pub fn new() -> Builder {
         Builder::default()
     }
 
     /// Use a specific android log buffer. Defaults to the main buffer
-    /// is used as tag (if present)
+    /// is used as tag (if present).
     ///
     /// # Examples
     ///
@@ -270,7 +247,6 @@ impl<'a> Builder<'a> {
     /// # use android_logd_logger::Buffer;
     ///
     /// let mut builder = Builder::new();
-    ///
     /// builder.buffer(Buffer::Crash)
     ///     .init();
     /// ```
@@ -280,7 +256,7 @@ impl<'a> Builder<'a> {
     }
 
     /// Use a specific log tag. If no tag is set the module path
-    /// is used as tag (if present)
+    /// is used as tag (if present).
     ///
     /// # Examples
     ///
@@ -292,13 +268,12 @@ impl<'a> Builder<'a> {
     /// builder.tag("foo")
     ///     .init();
     /// ```
-    pub fn tag(&mut self, tag: &'a str) -> &mut Self {
-        self.tag = Some(tag);
+    pub fn tag(&mut self, tag: &str) -> &mut Self {
+        self.tag = TagMode::Custom(tag.to_string());
         self
     }
 
-    /// Prepend module to log message. If set true the Rust module path
-    /// is prepended to the log message.
+    /// Use the target string as tag
     ///
     /// # Examples
     ///
@@ -306,12 +281,42 @@ impl<'a> Builder<'a> {
     /// # use android_logd_logger::Builder;
     ///
     /// let mut builder = Builder::new();
-    ///
-    /// builder.prepend_module(true)
-    ///     .init();
+    /// builder.tag_target().init();
     /// ```
-    pub fn prepend_module(&mut self, prepend: bool) -> &mut Self {
-        self.prepend_module = Some(prepend);
+    pub fn tag_target(&mut self) -> &mut Self {
+        self.tag = TagMode::Target;
+        self
+    }
+
+    /// Use the target string as tag and strip off ::.*
+    ///
+    /// # Examples
+    ///
+    /// ```
+    /// # use android_logd_logger::Builder;
+    ///
+    /// let mut builder = Builder::new();
+    /// builder.tag_target_strip().init();
+    /// ```
+    pub fn tag_target_strip(&mut self) -> &mut Self {
+        self.tag = TagMode::TargetStrip;
+        self
+    }
+
+    /// Prepend module to log message.
+    ///
+    /// If set true the Rust module path is prepended to the log message.
+    ///
+    /// # Examples
+    ///
+    /// ```
+    /// # use android_logd_logger::Builder;
+    ///
+    /// let mut builder = Builder::new();
+    /// builder.prepend_module(true).init();
+    /// ```
+    pub fn prepend_module(&mut self, prepend_module: bool) -> &mut Self {
+        self.prepend_module = prepend_module;
         self
     }
 
@@ -327,7 +332,7 @@ impl<'a> Builder<'a> {
     ///
     /// let mut builder = Builder::new();
     ///
-    /// builder.filter_module("path::to::module", LevelFilter::Info);
+    /// builder.filter_module("path::to::module", LevelFilter::Info).init();
     /// ```
     pub fn filter_module(&mut self, module: &str, level: LevelFilter) -> &mut Self {
         self.filter.filter_module(module, level);
@@ -345,8 +350,7 @@ impl<'a> Builder<'a> {
     /// # use android_logd_logger::Builder;
     ///
     /// let mut builder = Builder::new();
-    ///
-    /// builder.filter_level(LevelFilter::Info);
+    /// builder.filter_level(LevelFilter::Info).init();
     /// ```
     pub fn filter_level(&mut self, level: LevelFilter) -> &mut Self {
         self.filter.filter_level(level);
@@ -367,8 +371,7 @@ impl<'a> Builder<'a> {
     /// # use android_logd_logger::Builder;
     ///
     /// let mut builder = Builder::new();
-    ///
-    /// builder.filter(Some("path::to::module"), LevelFilter::Info);
+    /// builder.filter(Some("path::to::module"), LevelFilter::Info).init();
     /// ```
     pub fn filter(&mut self, module: Option<&str>, level: LevelFilter) -> &mut Self {
         self.filter.filter(module, level);
@@ -423,25 +426,26 @@ impl<'a> Builder<'a> {
     fn build(&mut self) -> Logger {
         let buffer = self.buffer.unwrap_or(Buffer::Main);
         let filter = self.filter.build();
-        let prepend_module = self.prepend_module.unwrap_or(false);
-        Logger::new(buffer, filter, self.tag.map(str::to_string), prepend_module).expect("Failed to build logger")
+        let prepend_module = self.prepend_module;
+        Logger::new(buffer, filter, self.tag.clone(), prepend_module).expect("Failed to build logger")
     }
 }
 
 struct Logger {
     filter: Filter,
-    tag: Option<String>,
+    tag: TagMode,
     prepend_module: bool,
-    _buffer_id: Buffer,
+    #[allow(unused)]
+    buffer_id: Buffer,
 }
 
 impl Logger {
-    pub fn new(buffer_id: Buffer, filter: Filter, tag: Option<String>, prepend_module: bool) -> Result<Logger, io::Error> {
+    pub fn new(buffer_id: Buffer, filter: Filter, tag: TagMode, prepend_module: bool) -> Result<Logger, io::Error> {
         Ok(Logger {
             filter,
             tag,
             prepend_module,
-            _buffer_id: buffer_id,
+            buffer_id,
         })
     }
 }
@@ -459,11 +463,7 @@ impl Log for Logger {
         let args = record.args().to_string();
         let message = if let Some(module_path) = record.module_path() {
             if self.prepend_module {
-                let mut message = String::with_capacity(module_path.len() + args.len());
-                message.push_str(module_path);
-                message.push_str(": ");
-                message.push_str(&args);
-                message
+                [module_path, &args].join(": ")
             } else {
                 args
             }
@@ -472,55 +472,26 @@ impl Log for Logger {
         };
 
         let priority: Priority = record.metadata().level().into();
+        let tag = match &self.tag {
+            TagMode::Target => record.target(),
+            TagMode::TargetStrip => record
+                .target()
+                .split_once("::")
+                .map(|(tag, _)| tag)
+                .unwrap_or_else(|| record.target()),
+            TagMode::Custom(tag) => tag.as_str(),
+        };
 
         #[cfg(target_os = "android")]
-        {
-            let timestamp = SystemTime::now();
-
-            let tag = if let Some(ref tag) = self.tag {
-                tag
-            } else {
-                record.module_path().unwrap_or_default()
-            };
-
-            let tag_len = tag.bytes().len() + 1;
-            let message_len = message.bytes().len() + 1;
-
-            let mut buffer = bytes::BytesMut::with_capacity(12 + tag_len + message_len);
-
-            buffer.put_u8(self._buffer_id.into());
-            buffer.put_u16_le(thread::id() as u16);
-            let timestamp = timestamp
-                .duration_since(std::time::UNIX_EPOCH)
-                .expect("Failed to aquire time");
-            buffer.put_u32_le(timestamp.as_secs() as u32);
-            buffer.put_u32_le(timestamp.subsec_nanos());
-            buffer.put_u8(priority as u8);
-            buffer.put(tag.as_bytes());
-            buffer.put_u8(0);
-
-            buffer.put(message.as_bytes());
-            buffer.put_u8(0);
-
-            #[cfg(feature = "tls")]
-            SOCKET.with(|f| f.send(&buffer).expect("Logd socket error"));
-
-            #[cfg(not(feature = "tls"))]
-            SOCKET.send(&buffer).expect("Logd socket error");
-        }
+        logd::log(tag, self.buffer_id, priority, &message);
 
         #[cfg(not(target_os = "android"))]
         {
-            let datetime = ::time::OffsetDateTime::now_utc();
-            let tag = if let Some(ref tag) = self.tag {
-                tag
-            } else {
-                record.module_path().unwrap_or_default()
-            };
+            let now = ::time::OffsetDateTime::now_utc();
             println!(
                 "{}.{} {} {} {} {}: {}",
-                datetime.format("%Y-%m-%d %T"),
-                &datetime.format("%N")[..3],
+                now.format("%Y-%m-%d %T"),
+                &now.format("%N")[..3],
                 std::process::id(),
                 thread::id(),
                 priority,
@@ -547,7 +518,7 @@ impl Log for Logger {
 /// Additionally it is possible to set whether the modul path appears in a log message.
 ///
 /// After a call to [`init`](Builder::init) the global logger is initialized with the configuration.
-pub fn builder<'a>() -> Builder<'a> {
+pub fn builder() -> Builder {
     Builder::default()
 }
 
@@ -751,23 +722,7 @@ pub fn write_event_buffer(log_buffer: Buffer, event: &Event) -> Result<(), Error
     }
 
     #[cfg(target_os = "android")]
-    {
-        let mut buffer = bytes::BytesMut::with_capacity(LOGGER_ENTRY_MAX_LEN);
-        let timestamp = event.timestamp.duration_since(std::time::UNIX_EPOCH).unwrap();
-
-        buffer.put_u8(log_buffer.into());
-        buffer.put_u16_le(thread::id() as u16);
-        buffer.put_u32_le(timestamp.as_secs() as u32);
-        buffer.put_u32_le(timestamp.subsec_nanos());
-        buffer.put_u32_le(event.tag);
-        buffer.put(event.value.as_bytes());
-
-        #[cfg(feature = "tls")]
-        SOCKET.with(|f| f.send(&buffer).map_err(Error::Io))?;
-
-        #[cfg(not(feature = "tls"))]
-        SOCKET.send(&buffer).map_err(Error::Io)?;
-    }
+    logd::write_event(log_buffer, event);
 
     #[cfg(not(target_os = "android"))]
     println!("buffer: {:?}, event: {:?}", log_buffer, event);
